@@ -51,12 +51,12 @@
   "Run `git gui' for the current git repository."
   (interactive)
   (magit-with-toplevel
-    (call-process magit-git-executable nil 0 nil "gui")))
+    (magit-process-file magit-git-executable nil 0 nil "gui")))
 
 ;;;###autoload
 (defun magit-run-git-gui-blame (commit filename &optional linenum)
   "Run `git gui blame' on the given FILENAME and COMMIT.
-Interactively run it for the current file and the HEAD, with a
+Interactively run it for the current file and the `HEAD', with a
 prefix or when the current file cannot be determined let the user
 choose.  When the current buffer is visiting FILENAME instruct
 blame to center around the line point is on."
@@ -65,15 +65,15 @@ blame to center around the line point is on."
      (when (or current-prefix-arg
                (not (setq revision "HEAD"
                           filename (magit-file-relative-name nil 'tracked))))
-       (setq revision (magit-read-branch-or-commit "Blame from revision")
-             filename (magit-read-file-from-rev revision "Blame file")))
+       (setq revision (magit-read-branch-or-commit "Blame from revision"))
+       (setq filename (magit-read-file-from-rev revision "Blame file")))
      (list revision filename
            (and (equal filename
                        (ignore-errors
                          (magit-file-relative-name buffer-file-name)))
                 (line-number-at-pos)))))
   (magit-with-toplevel
-    (apply #'call-process magit-git-executable nil 0 nil "gui" "blame"
+    (apply #'magit-process-file magit-git-executable nil 0 nil "gui" "blame"
            `(,@(and linenum (list (format "--line=%d" linenum)))
              ,commit
              ,filename))))
@@ -82,19 +82,19 @@ blame to center around the line point is on."
 (defun magit-run-gitk ()
   "Run `gitk' in the current repository."
   (interactive)
-  (call-process magit-gitk-executable nil 0))
+  (magit-process-file magit-gitk-executable nil 0))
 
 ;;;###autoload
 (defun magit-run-gitk-branches ()
   "Run `gitk --branches' in the current repository."
   (interactive)
-  (call-process magit-gitk-executable nil 0 nil "--branches"))
+  (magit-process-file magit-gitk-executable nil 0 nil "--branches"))
 
 ;;;###autoload
 (defun magit-run-gitk-all ()
   "Run `gitk --all' in the current repository."
   (interactive)
-  (call-process magit-gitk-executable nil 0 nil "--all"))
+  (magit-process-file magit-gitk-executable nil 0 nil "--all"))
 
 ;;; Emacs Tools
 
@@ -124,7 +124,7 @@ like pretty much every other keymap:
 (defun magit-dired-jump (&optional other-window)
   "Visit file at point using Dired.
 With a prefix argument, visit in another window.  If there
-is no file at point then instead visit `default-directory'."
+is no file at point, then instead visit `default-directory'."
   (interactive "P")
   (dired-jump other-window (-if-let (file (magit-file-at-point))
                                (progn (setq file (expand-file-name file))
@@ -229,8 +229,8 @@ a position in a file-visiting buffer."
   (let (buf pos)
     (save-window-excursion
       (call-interactively #'magit-diff-visit-file)
-      (setq buf (current-buffer)
-            pos (point)))
+      (setq buf (current-buffer))
+      (setq pos (point)))
     (save-excursion
       (with-current-buffer buf
         (goto-char pos)
@@ -247,5 +247,19 @@ on a position in a file-visiting buffer."
                           (prompt-for-change-log-name))))
   (magit-add-change-log-entry whoami file-name t))
 
+;;; Miscellaneous
+
+;;;###autoload
+(defun magit-abort-dwim ()
+  "Abort current operation.
+Depending on the context, this will abort a merge, a rebase, a
+patch application, a cherry-pick, a revert, or a bisect."
+  (interactive)
+  (cond ((magit-merge-state)             (magit-merge-abort))
+        ((magit-rebase-in-progress-p)    (magit-rebase-abort))
+        ((magit-am-in-progress-p)        (magit-am-abort))
+        ((magit-sequencer-in-progress-p) (magit-sequencer-abort))
+        ((magit-bisect-in-progress-p)    (magit-bisect-reset))))
+
 (provide 'magit-extras)
 ;;; magit-extras.el ends here
