@@ -12,7 +12,7 @@
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
 
-;; Package-Requires: ((emacs "24.4") (async "1.9") (dash "2.13.0"))
+;; Package-Requires: ((emacs "24.4") (async "1.9.2") (dash "2.13.0"))
 ;; Keywords: bindings
 ;; Homepage: https://github.com/magit/magit
 
@@ -200,7 +200,7 @@ The prefix used to toggle any switch can be changed by binding
 another key to `magit-invoke-popup-switch'.  Likewise binding
 another key to `magit-invoke-popup-option' changes the prefixed
 used to set any option.  The two prefixes have to be different.
-If you change these bindings you should also change the `prefix'
+If you change these bindings, you should also change the `prefix'
 property of the button types `magit-popup-switch-button' and
 `magit-popup-option-button'.
 
@@ -313,7 +313,7 @@ Don't confuse this with `magit-current-popup-args'.")
 Use this inside the `interactive' form of a popup aware command
 to determine whether it was invoked from a popup and if so from
 which popup.  If the current command was invoked without the use
-of a popup then this is nil.")
+of a popup, then this is nil.")
 
 (defvar magit-current-popup-action nil
   "The popup action now being executed.")
@@ -348,10 +348,10 @@ only arguments that don't match any of the regexps are returned,
 or `:only' which doesn't change the behaviour."
   (let ((-compare-fn (lambda (a b) (magit-popup-arg-match b a))))
     (-filter (if (eq (car filter) :not)
-                 (lambda (arg) (not (-contains? (cdr filter) arg)))
+                 (lambda (arg) (not (-contains-p (cdr filter) arg)))
                (when (eq (car filter) :only)
                  (pop filter))
-               (lambda (arg) (-contains? filter arg)))
+               (lambda (arg) (-contains-p filter arg)))
              magit-current-popup-args)))
 
 (defun magit-popup-arg-match (pattern string)
@@ -441,7 +441,7 @@ when the command is invoked directly, then it returns the default
 value of the variable `SHORTNAME-arguments'.
 
 Optional argument GROUP specifies the Custom group into which the
-option is placed.  If omitted then the option is placed into some
+option is placed.  If omitted, then the option is placed into some
 group the same way it is done when directly using `defcustom' and
 omitting the group, except when NAME begins with \"magit-\", in
 which case the group `magit-git-arguments' is used.
@@ -491,7 +491,9 @@ usually specified in that order):
   Controls when to display the popup buffer and when to invoke
   the default action (if any) directly.  This overrides the
   global default set using `magit-popup-use-prefix-argument'.
-  The value, if specified, should be one of `default' or `popup'.
+  The value, if specified, should be one of `default' or `popup',
+  or a function that is called with no arguments and returns one
+  of these symbols.
 
 `:max-action-columns'
   The maximum number of actions to display on a single line, a
@@ -589,11 +591,11 @@ changed in `magit-popup-mode-keymap').
 DESC is a string describing the purpose of the argument, it is
 displayed in the popup.
 
-If optional ENABLE is non-nil then the switch is on by default.
+If optional ENABLE is non-nil, then the switch is on by default.
 
 SWITCH is inserted after all other switches already defined for
 POPUP, unless optional PREPEND is non-nil, in which case it is
-placed first.  If optional AT is non-nil then it should be the
+placed first.  If optional AT is non-nil, then it should be the
 KEY of another switch already defined for POPUP, the argument
 is then placed before or after AT, depending on PREPEND."
   (declare (indent defun))
@@ -617,9 +619,14 @@ displayed in the popup.
 If optional VALUE is non-nil then the option is on by default,
 and VALUE is its default value.
 
+READER is used to read a value from the user when the option is
+invoked and does not currently have a value.  It should take one
+argument and use it as the prompt.  If this is nil, then
+`read-from-minibuffer' is used.
+
 OPTION is inserted after all other options already defined for
 POPUP, unless optional PREPEND is non-nil, in which case it is
-placed first.  If optional AT is non-nil then it should be the
+placed first.  If optional AT is non-nil, then it should be the
 KEY of another option already defined for POPUP, the argument
 is then placed before or after AT, depending on PREPEND."
   (declare (indent defun))
@@ -643,7 +650,7 @@ actually used for anything.
 
 COMMAND is inserted after all other commands already defined for
 POPUP, unless optional PREPEND is non-nil, in which case it is
-placed first.  If optional AT is non-nil then it should be the
+placed first.  If optional AT is non-nil, then it should be the
 KEY of another command already defined for POPUP, the command
 is then placed before or after AT, depending on PREPEND."
   (declare (indent defun))
@@ -665,7 +672,7 @@ displayed in the popup.
 
 COMMAND is inserted after all other commands already defined for
 POPUP, unless optional PREPEND is non-nil, in which case it is
-placed first.  If optional AT is non-nil then it should be the
+placed first.  If optional AT is non-nil, then it should be the
 KEY of another command already defined for POPUP, the command
 is then placed before or after AT, depending on PREPEND."
   (declare (indent defun))
@@ -763,6 +770,9 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
          (val     (symbol-value (plist-get def :variable)))
          (default (plist-get def :default-action))
          (local   (plist-get def :use-prefix))
+         (local   (if (functionp local)
+                      (funcall local)
+                    local))
          (use-prefix (or local magit-popup-use-prefix-argument)))
     (cond
      ((or (and (eq use-prefix 'default) arg)
@@ -826,22 +836,23 @@ TYPE is one of `:action', `:sequence-action', `:switch', or
     (when (and variable (not (magit-popup-event-arg variable)))
       (setq action variable)
       (setq variable nil))
-    (if (or action variable)
-        (let* ((magit-current-popup magit-this-popup)
-               (magit-current-popup-args (magit-popup-get-args))
-               (command (magit-popup-event-fun (or action variable)))
-               (magit-current-popup-action command))
-          (when action
-            (magit-popup-quit))
-          (call-interactively command)
-          (setq this-command command)
-          (unless action
-            (magit-refresh-popup-buffer)))
-      (if (eq event ?q)
-          (progn (magit-popup-quit)
-                 (when magit-previous-popup
-                   (magit-popup-mode-setup magit-previous-popup nil)))
-        (user-error "%c isn't bound to any action" event)))))
+    (cond ((or action variable)
+           (let* ((magit-current-popup magit-this-popup)
+                  (magit-current-popup-args (magit-popup-get-args))
+                  (command (magit-popup-event-fun (or action variable)))
+                  (magit-current-popup-action command))
+             (when action
+               (magit-popup-quit))
+             (call-interactively command)
+             (setq this-command command)
+             (unless action
+               (magit-refresh-popup-buffer))))
+          ((eq event ?q)
+           (magit-popup-quit)
+           (when magit-previous-popup
+             (magit-popup-mode-setup magit-previous-popup nil)))
+          (t
+           (user-error "%c isn't bound to any action" event)))))
 
 (defun magit-popup-set-variable
     (variable choices &optional default other)
@@ -1272,8 +1283,8 @@ variable whose value may be used as a default."
 (defun magit-popup-export-file-args (args)
   (let ((files (--first (string-prefix-p "-- " it) args)))
     (when files
-      (setq args  (remove files args)
-            files (split-string (substring files 3) ",")))
+      (setq args  (remove files args))
+      (setq files (split-string (substring files 3) ",")))
     (list args files)))
 
 (defconst magit-popup-font-lock-keywords
