@@ -24,7 +24,7 @@
 ;;; Commentary:
 
 ;; This library implements "sections" as used in all Magit buffers.
-;; If you have used Magit before then you probably know what that
+;; If you have used Magit before, then you probably know what that
 ;; means, otherwise think "read-only Org-Mode for Git", kinda.
 
 ;;; Code:
@@ -98,7 +98,7 @@ diff-related sections being the only exception."
     magit-section-set-visibility-from-cache)
   "Hook used to set the initial visibility of a section.
 Stop at the first function that returns non-nil.  The value
-should be `show' or `hide'.  If no function returns non-nil
+should be `show' or `hide'.  If no function returns non-nil,
 determine the visibility as usual, i.e. use the hardcoded
 section specific default (see `magit-insert-section')."
   :package-version '(magit . "2.4.0")
@@ -352,7 +352,7 @@ With a prefix argument also expand it." heading)
   (interactive (list (magit-current-section)))
   (goto-char (magit-section-start section))
   (let* ((children (magit-section-children section))
-         (show (-any? 'magit-section-hidden children)))
+         (show (-any-p 'magit-section-hidden children)))
     (dolist (c children)
       (setf (magit-section-hidden c) show)))
   (magit-section-show section))
@@ -402,10 +402,10 @@ hidden."
       (progn (magit-section-show section)
              (magit-section-hide-children section))
     (let ((children (magit-section-children section)))
-      (cond ((and (-any? 'magit-section-hidden   children)
-                  (-any? 'magit-section-children children))
+      (cond ((and (-any-p 'magit-section-hidden   children)
+                  (-any-p 'magit-section-children children))
              (magit-section-show-headings section))
-            ((-any? 'magit-section-hidden-body children)
+            ((-any-p 'magit-section-hidden-body children)
              (magit-section-show-children section))
             (t
              (magit-section-hide section))))))
@@ -414,10 +414,10 @@ hidden."
   "Cycle visibility of all sections in the current buffer."
   (interactive)
   (let ((children (magit-section-children magit-root-section)))
-    (cond ((and (-any? 'magit-section-hidden   children)
-                (-any? 'magit-section-children children))
+    (cond ((and (-any-p 'magit-section-hidden   children)
+                (-any-p 'magit-section-children children))
            (magit-section-show-headings magit-root-section))
-          ((-any? 'magit-section-hidden-body children)
+          ((-any-p 'magit-section-hidden-body children)
            (magit-section-show-children magit-root-section))
           (t
            (mapc 'magit-section-hide children)))))
@@ -437,22 +437,22 @@ hidden."
                     ((derived-mode-p 'magit-diff-mode)
                      (--filter (eq (magit-section-type it) 'file)
                                (magit-section-children magit-root-section)))))
-    (if (-any? 'magit-section-hidden sections)
+    (if (-any-p 'magit-section-hidden sections)
         (dolist (s sections)
           (magit-section-show s)
           (magit-section-hide-children s))
       (let ((children (-mapcat 'magit-section-children sections)))
-        (cond ((and (-any? 'magit-section-hidden   children)
-                    (-any? 'magit-section-children children))
+        (cond ((and (-any-p 'magit-section-hidden   children)
+                    (-any-p 'magit-section-children children))
                (mapc 'magit-section-show-headings sections))
-              ((-any? 'magit-section-hidden-body children)
+              ((-any-p 'magit-section-hidden-body children)
                (mapc 'magit-section-show-children sections))
               (t
                (mapc 'magit-section-hide sections)))))))
 
 (defun magit-section-hidden-body (section &optional pred)
   (--if-let (magit-section-children section)
-      (funcall (or pred '-any?) 'magit-section-hidden-body it)
+      (funcall (or pred '-any-p) 'magit-section-hidden-body it)
     (and (magit-section-content section)
          (magit-section-hidden  section))))
 
@@ -466,7 +466,7 @@ SECTION's body (and heading) obviously cannot be visible."
 
 (defun magit-section-show-level (level)
   "Show surrounding sections up to LEVEL.
-If LEVEL is negative show up to the absolute value.
+If LEVEL is negative, show up to the absolute value.
 Sections at higher levels are hidden."
   (if (< level 0)
       (let ((s (magit-current-section)))
@@ -588,13 +588,13 @@ precise."
              (magit-section-match-2 (cdr l1) (cdr l2))))))
 
 (defmacro magit-section-when (condition &rest body)
-  "If the section at point matches CONDITION evaluate BODY.
+  "If the section at point matches CONDITION, evaluate BODY.
 
 If the section matches, then evaluate BODY forms sequentially
 with `it' bound to the section and return the value of the last
 form.  If there are no BODY forms, then return the value of the
 section.  If the section does not match or if there is no section
-at point then return nil.
+at point, then return nil.
 
 See `magit-section-match' for the forms CONDITION can take."
   (declare (indent 1)
@@ -616,7 +616,7 @@ section is compared against each CONDITION; the BODY forms of the
 first match are evaluated sequentially and the value of the last
 form is returned.  Inside BODY the symbol `it' is bound to the
 section at point.  If no clause succeeds or if there is no
-section at point return nil.
+section at point, return nil.
 
 See `magit-section-match' for the forms CONDITION can take.
 Additionally a CONDITION of t is allowed in the final clause, and
@@ -770,7 +770,7 @@ The `content' property of the section struct is the end of the
 heading (which lasts from `start' to `content') and the beginning
 of the the body (which lasts from `content' to `end').  If the
 value of `content' is nil, then the section has no heading and
-its body cannot be collapsed.  If a section does have a heading
+its body cannot be collapsed.  If a section does have a heading,
 then its height must be exactly one line, including a trailing
 newline character.  This isn't enforced, you are responsible for
 getting it right.  The only exception is that this function does
@@ -786,7 +786,8 @@ insert a newline character if necessary."
   (magit-maybe-make-margin-overlay)
   (setf (magit-section-content magit-insert-section--current) (point-marker)))
 
-(defvar magit-insert-headers-hook nil "For internal use only.")
+(defvar magit-insert-headers--hook nil "For internal use only.")
+(defvar magit-insert-headers--beginning nil "For internal use only.")
 
 (defun magit-insert-headers (hooks)
   (let ((magit-insert-section-hook
@@ -794,18 +795,19 @@ insert a newline character if necessary."
                (if (listp magit-insert-section-hook)
                    magit-insert-section-hook
                  (list magit-insert-section-hook))))
-        (magit-insert-headers-hook hooks)
+        (magit-insert-headers--hook hooks)
         wrapper)
-    (while (and (setq wrapper (pop magit-insert-headers-hook))
-                (= (point) (point-min)))
+    (setq magit-insert-headers--beginning (point))
+    (while (and (setq wrapper (pop magit-insert-headers--hook))
+                (= (point) magit-insert-headers--beginning))
       (funcall wrapper))))
 
 (defun magit-insert-remaining-headers ()
-  (if (= (point) (point-min))
+  (if (= (point) magit-insert-headers--beginning)
       (magit-cancel-section)
     (magit-insert-heading)
     (remove-hook 'magit-insert-section-hook 'magit-insert-remaining-headers)
-    (mapc #'funcall magit-insert-headers-hook)
+    (mapc #'funcall magit-insert-headers--hook)
     (insert "\n")))
 
 (defun magit-insert-child-count (section)
@@ -885,7 +887,7 @@ highlighted using `magit-diff-highlight'.  Return t."
 
 (defun magit-section-highlight-selection (_ selection)
   "Highlight the section-selection region.
-If SELECTION is non-nil then it is a list of sections selected by
+If SELECTION is non-nil, then it is a list of sections selected by
 the region.  The headings of these sections are then highlighted.
 
 This is a fallback for people who don't want to highlight the
@@ -1032,9 +1034,9 @@ either `hide' or nil."
 (defun magit-section-siblings (section &optional direction)
   "Return a list of the sibling sections of SECTION.
 
-If optional DIRECTION is `prev' then return siblings that come
-before SECTION, if it is `next' then return siblings that come
-after SECTION.  For all other values return all siblings
+If optional DIRECTION is `prev', then return siblings that come
+before SECTION.  If it is `next', then return siblings that come
+after SECTION.  For all other values, return all siblings
 excluding SECTION itself."
   (-when-let (parent (magit-section-parent section))
     (let ((siblings  (magit-section-children parent)))
@@ -1072,7 +1074,7 @@ apply to diffs where things get a bit more complicated, but even
 here if the region looks like it usually does, then that's not
 a valid selection as far as this function is concerned.
 
-If optional TYPES is non-nil then the selection not only has to
+If optional TYPES is non-nil, then the selection not only has to
 be valid; the types of all selected sections additionally have to
 match one of TYPES, or nil is returned."
   (when (use-region-p)
@@ -1113,6 +1115,27 @@ If optional SECTION is nil, use the current section."
                 (magit-section-position-in-heading-p section (region-end))))
        t))
 
+(defun magit-section--backward-protected ()
+  "Move to the beginning of the current or the previous visible section.
+Same as `magit-section-backward' but for non-interactive use.
+Suppress `magit-section-movement-hook', and return a boolean to
+indicate whether a section was found, instead of raising an error
+if not."
+  (condition-case nil
+      (let ((magit-section-movement-hook nil))
+        (magit-section-backward)
+        t)
+    (user-error nil)))
+
+(defun magit-section--backward-find (predicate)
+  "Move to the first previous section satisfying PREDICATE.
+PREDICATE does not take any parameter and should not move
+point."
+  (let (found)
+    (while (and (setq found (magit-section--backward-protected))
+                (not (funcall predicate))))
+    found))
+
 (defun magit-wash-sequence (function)
   "Repeatedly call FUNCTION until it returns nil or eob is reached.
 FUNCTION has to move point forward or return nil."
@@ -1123,7 +1146,7 @@ FUNCTION has to move point forward or return nil."
 
 Add FUNCTION at the beginning of the hook list unless optional
 APPEND is non-nil, in which case FUNCTION is added at the end.
-If FUNCTION already is a member then move it to the new location.
+If FUNCTION already is a member, then move it to the new location.
 
 If optional AT is non-nil and a member of the hook list, then
 add FUNCTION next to that instead.  Add before or after AT, or
@@ -1175,5 +1198,20 @@ again use `remove-hook'."
         (set hook value)
       (set-default hook value))))
 
+(defun magit-run-section-hook (hook)
+  "Run HOOK, warning about invalid entries."
+  (--if-let (-remove #'functionp (symbol-value hook))
+      (progn
+        (message "`%s' contains entries that are no longer valid.
+%s\nUsing standard value instead.  Please re-configure hook variable."
+                 hook
+                 (mapconcat (lambda (sym) (format "  `%s'" sym)) it "\n"))
+        (sit-for 5)
+        (defvar magit--hook-standard-value nil)
+        (let ((magit--hook-standard-value
+               (eval (car (get hook 'standard-value)))))
+          (run-hooks 'magit---hook-standard-value)))
+    (run-hooks hook)))
+
 (provide 'magit-section)
 ;;; magit-section.el ends here
